/*  FFTGRAF Benchmark Version 1 Copyrigh (C) Roy Longbottom and Scott Taylor 2001 - 2004
 *    
 *    Version 1 Mar 2001 
 *    Version 1.01 Apr 2001, added compare, no default graph
 *    Version 1.02 May 2001, added compare text output
 *    Version 1.1  Jan 2002, different display method
 * 
 *    Scott Taylor's optimised code
 *
 *    See FFTGraf3.cpp for general details. Main differences are FFT
 *    calculations in fftSP() and fftDP()
 *     
 */

 
#define NAME "Test"
#define TITLE " FFT Graph Benchmark "
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <direct.h>
#include <commdlg.h>

#include "FFTGraf.h"


// ************* GLOBAL VARIABLES **********

#define SIZE  1048576  

double   cDataDP[SIZE][2];
double   cExpDP[SIZE/2][2];
float    cDataSP[SIZE][2];
float    cExpSP[SIZE/2][2];
int      table[SIZE];

#define I       0
#define Q       1
 

float  PISP = (float)3.14159265;
double PIDP = 3.14159265358979323846;

BOOL bQuit = FALSE;

HWND   mainHWND;

HINSTANCE  MyInstance;
RECT    clientRect;


int     numberFFTs[11];
double  miniSP[11];
double  miniDP[11];
double  miniSPA[11];
double  miniDPA[11];
double  miniSPB[11];
double  miniDPB[11];
int     minFFTRun;
int     FFTsizes = 11;
int     maxFFTRun;
int     FFTpasses = 5;
int     maxruns = 12;
int     maxSecs = 15;

char   currentDir[MAX_PATH];

char   fileMessage[256];

char    logfile[256] = "FFTGraf.txt";
char    bmpfile[256] = "FFTGraf.bmp";

char    commandLine[1024] = "";
char    checkSP[80];
char    checkDP[80];

int  screenWide = 640;
int  screenHigh = 480;
double SPFFT[11][20];
double DPFFT[11][20];
double SP1[11][20];
double SP2[11][20];
double DP1[11][20];
double DP2[11][20];
double SPres;
double DPres;

BOOL   testRunning = FALSE;
RECT   recwin;
int    xtraWide; 

char   timeday1[30];
char   grafFile[256];
char   fileToRead[256];
BOOL    displayOnly = FALSE;
WIN32_FIND_DATA W32DiskData;
BOOL  loadIt;

int   doRun = 0;
int   tpass = 0;
int   jpass = 0;

int   butT = 365;
int   butL = 380;
int   butW = 80;
int   butH = 22;
int  flash1 = 0;
int  flash2 = 0;
int  runningTest = 0;
int  ramMB;
int  ramKB;
int  nSP;
int  nDP;
short  keyPress;
BOOL   escPressed = FALSE;
char  amsg[100];
char version[50] = " Version 1.1";
double maxScale = 0;
double maxVal[40];
int maxScalePos;
BOOL  compareTwo = FALSE;
char config1[200];
char config2[200];
char config3[200];

//  ************** PROTOPTYPES *****************

BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);
static BOOL InitApp(HINSTANCE, LPSTR, int);
long FAR PASCAL WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);

BOOL FAR PASCAL WriteReadGraph( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL UpdateCombos( HWND, int, int);

BOOL  reView();
BOOL  drawGraphs(int, int, HDC);

BOOL  saveGrafTxt();
BOOL  saveBmp();
BOOL loadFile();
BOOL compareTxt();
static BOOL GetFileName();

void bitrev(int, int);
void createExpTableSP(int n, int sign);
void createExpTableDP(int n, int sign);
void createRealDataSP(int n, float synthFreq);
void createRealDataDP(int n, double synthFreq);
void fftSP(int t, int n); 
void fftDP(int t, int n);
void SPCalculate();
void DPCalculate();
void Compare0();
void Compare1();
void Compare2();
void checkNoiseSP();
void checkNoiseDP();


/*
 * WinMain - initialization, message loop
 */
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow)
{
    MSG         msg;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;
      
    if (maxFFTRun < 10)
    {
        sprintf(amsg, "RAM %dMB, maximum size FFT reduced to %dK",
                          ramMB, numberFFTs[maxFFTRun]);
        MessageBox(NULL, amsg, "Not enough RAM", MB_ICONWARNING | MB_OK);
    }
 
    while(!bQuit)
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else if (doRun > 0)
        {
            maxScale = 0;
            if (doRun == 1)
            {
                SPCalculate();
                checkNoiseSP();
            }
            else
            {
                DPCalculate();
                checkNoiseDP();
            }
            jpass = jpass + 1;
            if (escPressed)
            {
                jpass = FFTpasses;
                tpass = maxFFTRun + 1;
            }   
            if (jpass == FFTpasses)
            {
                jpass = 0;
                tpass = tpass + 1;
            }
            if (tpass > maxFFTRun - minFFTRun)
            {
                if (doRun == 1 && !escPressed)
                {
                    doRun = 2;
                    flash1 = 0;
                    flash2 = 0;
                    tpass = 0;
                }
                else
                {
                    doRun = 0;
                    if (!escPressed)
                    {
                        saveGrafTxt();
                    }
                    escPressed = FALSE;
                    testRunning = FALSE;
                    flash1 = 0;
                    flash2 = 0;
                    reView();
                }
            }
        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }

    return msg.wParam;
    
} /* WinMain */

/****************************************************************************/
/*                   Initialization and object creation                     */
/****************************************************************************/
/*
 * InitApp
 * Creates window
 */
static BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND hwnd;
    WNDCLASS wc;
    LPCTSTR  dir = NULL;
    int i, j, gb;
    char *delims = {" ,"};
//    char commands[20][256];

    char buf[1024] = "";

    
    /*
     * set up and registers the window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;

    /*
     * Create the window
     */

    hwnd =
        CreateWindow(
            NAME,
            TITLE,
            WS_OVERLAPPEDWINDOW, // | WS_HSCROLL | WS_VSCROLL,
            0,
            0,
            screenWide,
            screenHigh,
            NULL, 
            NULL, 
            hInstance, 
            NULL 
        );

    if (!hwnd) return FALSE;

    mainHWND = hwnd;
    strcpy(commandLine, strupr(cmdline));

    WhatConfig();
    ramKB = ramsize / 1024;
    ramMB = ramsize / 1024 / 1024;
    GetCurrentDirectory(MAX_PATH, currentDir);

    maxFFTRun = 10;                 // max used  28 + 16 MB FFT 1024 KB
    if (ramMB < 50) maxFFTRun = 9;  // max used  14 +  8 MB FFT  512 KB
    if (ramMB < 25) maxFFTRun = 8;  // max used   7 +  4 MB FFT  256 KB
    if (ramMB < 15) maxFFTRun = 7;  // max used 3.5 +  2 MB FFT  128 KB

    minFFTRun = 0;

    gb = 1;
    for (i=0; i<FFTsizes; i++)
    {
        numberFFTs[i] = gb;
        gb = gb * 2;
        for (j=0; j<maxruns; j++)
        {
            SPFFT[i][j] = 0;
            DPFFT[i][j] = 0;
        }
    }
    for (i=0; i<10; i++)
    {
        maxVal[i] = 10.0 - (double)i;
        maxVal[i+9] = (10.0 - (double)i) / 10.0;
        maxVal[i+18] = (10.0 - (double)i) / 100.0;
        maxVal[i+27] = (10.0 - (double)i) / 1000.0;
    }        
    
    keyPress = GetAsyncKeyState(VK_ESCAPE);

    // Display the window
    
    ShowWindow(hwnd, cmdshow);    
    UpdateWindow(hwnd);

    return TRUE;
    
}  // InitApp


/*
 * WindowProc - handle messages for the main application window
 */

long FAR PASCAL WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
    static HINSTANCE hand;
    char   tempFile[256];
    PAINTSTRUCT      ps;
    HDC     hdc;
       
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
        break;
        
    case WM_CREATE:    
        CreateWindow("BUTTON", "Variables", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                     butL, butT, butW, butH, hwnd, (HMENU)PARAM_GRAPH, hand, NULL);
        CreateWindow("BUTTON", "Exit", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                     butL+butW, butT, butW, butH, hwnd, (HMENU)FINISH_GRAPH, hand, NULL);  
        CreateWindow("BUTTON", "Run", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                     butL+2*butW, butT, butW, butH, hwnd, (HMENU)RUN_BUTTON, hand, NULL);  
        return 0;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_COMMAND:
        switch( LOWORD( wparam ) )
        {
        case MENU_ABOUT:
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            break;

        case PARAM_GRAPH:
            DialogBox( MyInstance,"DIALOG_21", hwnd, (DLGPROC)WriteReadGraph );
            reView();                   
            break;

        case MENU_PARAM:
            DialogBox( MyInstance,"DIALOG_21", hwnd, (DLGPROC)WriteReadGraph );
            reView();                   
            break;

        case MENU_SCALE:
            DialogBox( MyInstance,"DIALOG_21", hwnd, (DLGPROC)WriteReadGraph );
            reView();                   
            break;

        case MENU_RUNGRAPH:
            compareTwo = FALSE;
            local_time();
            strcpy(timeday1, timeday);
            WhatConfig();
            ramKB = ramsize / 1024;
            displayOnly = FALSE;
            SetWindowText(mainHWND, " FFT Graph Benchmark  **** RUNNING ****");  
            doRun = 1;
            tpass = 0;
            jpass = 0;
            flash1 = 1;
            flash2 = 0;
            break;
            

        case MENU_LOAD:
            loadIt = TRUE;
            compareTwo = FALSE;
            if (GetFileName())
            {
                FindFirstFile(grafFile, &W32DiskData);
                strcpy (fileToRead, W32DiskData.cFileName);
                loadFile();
                SetCurrentDirectory(currentDir);
            }
            break;

        case MENU_COMPARE:
            loadIt = TRUE;
            compareTwo = FALSE;
            Compare0();
            if (GetFileName())
            {
                FindFirstFile(grafFile, &W32DiskData);
                strcpy (fileToRead, W32DiskData.cFileName);
                loadFile();
                Compare1();    
            }                
            if (GetFileName())
            {
                FindFirstFile(grafFile, &W32DiskData);
                strcpy (fileToRead, W32DiskData.cFileName);
                loadFile();
                Compare2();
                SetCurrentDirectory(currentDir);
            }
            break;
            
        case MENU_SAVE:
            loadIt = FALSE;
            strcpy(tempFile, bmpfile);
            if (GetFileName())
            {
                saveBmp();
                SetCurrentDirectory(currentDir);
                strcpy(bmpfile, tempFile);
            }
            break;

        case MENU_COMPAREOUT:
            if (!compareTwo)
            {
                MessageBox(NULL, "Select results to compare via menu",
                "Not in compare mode", MB_ICONWARNING | MB_OK);
            }
            else
            {
                compareTxt();
            }
            break;

        case RUN_BUTTON:    
            compareTwo = FALSE;
            local_time();
            strcpy(timeday1, timeday);
            WhatConfig();
            ramKB = ramsize / 1024;
            displayOnly = FALSE;
            SetWindowText(mainHWND, " FFT Graph Benchmark  **** RUNNING ****");  
            doRun = 1;
            tpass = 0;
            jpass = 0;
            flash1 = 1;
            flash2 = 0;
            break;
       
        case MENU_VIEWGRAPH:
            reView();
            break;

        case MENU_HELP:
            DialogBox( MyInstance,"DIALOG_44", hwnd, (DLGPROC)AboutDlgProc );                   
            break;
            
        case FINISH_GRAPH:
            CleanUpAndPostQuit();
            break;                

        case MENU_EXIT:
            CleanUpAndPostQuit();
            break;                
        }
        break;
   
        case WM_PAINT:
             hdc = BeginPaint(hwnd, &ps);
             drawGraphs(FFTsizes, FFTsizes, hdc);
             EndPaint(hwnd, &ps);
             break;
           
        case WM_KEYDOWN:
            break;

       case WM_ACTIVATE:
            break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} /* WindowProc */



/*
 * AboutDlgProc - processes messages for the about dialog.
 */
BOOL FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} /* AboutDlgProc */


/*
 * CleanUpAndPostQuit
 * Release all D3DRM objects, post a quit message and set the bQuit flag
 */
void
CleanUpAndPostQuit(void)
{
    bQuit = TRUE;
    return;
}



BOOL FAR PASCAL UpdateCombos( HWND hdlg, int minb, int maxb)
{
    minFFTRun = minb;
    maxFFTRun = maxb;

    if (maxScalePos > 0)
    {
        maxScale =  maxVal[maxScalePos - 1];
    }
    else
    {
        maxScale = 0;
    }

    if(minb > maxb)
    {
        maxFFTRun = minb;
    }
    SendDlgItemMessage(hdlg, MINFFTSIZE, CB_SETCURSEL, minFFTRun, 0L);
    SendDlgItemMessage(hdlg, MAXFFTSIZE, CB_SETCURSEL, maxFFTRun, 0L);
    SendDlgItemMessage(hdlg, MAXSCALE, CB_SETCURSEL, maxScalePos, 0L);

    return TRUE;
}

BOOL FAR PASCAL WriteReadGraph( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    HWND     hwndName;
    int     tempint;
    int     wTextWidth;
    char    temptext[256];
    BOOL     dataOK = TRUE;
    int     i, j, minb, maxb;
    char    wdata[10];
    BOOL    isChanged = FALSE;
    int     tempPasses = FFTpasses;

    minb = minFFTRun;
    maxb = maxFFTRun;
    
    if (maxScale == 0) maxScalePos = 0;
    
    switch( msg )
     {
       case WM_INITDIALOG:
        SetDlgItemInt(hdlg, FFT_PASSES,  FFTpasses, FALSE); 
        SetDlgItemInt(hdlg, MAXRUNSECS,  maxSecs, FALSE); 

        for (i=0; i<FFTsizes; i++)
        {
           sprintf(wdata,"%4d", numberFFTs[i]);
           SendDlgItemMessage(hdlg, MINFFTSIZE, CB_ADDSTRING, 0, (LPARAM) wdata);
           SendDlgItemMessage(hdlg, MAXFFTSIZE, CB_ADDSTRING, 0, (LPARAM) wdata);
        }
        sprintf(wdata,"N/A");
        SendDlgItemMessage(hdlg, MAXSCALE, CB_ADDSTRING, 0, (LPARAM) wdata);
        for (i=0; i<37; i++)
        {
            sprintf(wdata,"%6.3f", maxVal[i]);
            SendDlgItemMessage(hdlg, MAXSCALE, CB_ADDSTRING, 0, (LPARAM) wdata);
        }
        UpdateCombos(hdlg, minb, maxb);


        hwndName = GetDlgItem(hdlg, GRAPH_FILE);
        SendMessage(hwndName, WM_SETTEXT, 0,
                                    (LPARAM) ((LPSTR) logfile));

        return( TRUE );

      case WM_COMMAND:
        if( LOWORD( wparam ) ==  DO_RUN)
        {
             hwndName = GetDlgItem(hdlg, GRAPH_FILE);
             wTextWidth = SendMessage(hwndName, WM_GETTEXT,
                      sizeof(temptext), (LPARAM) ((LPSTR) temptext));
             if (wTextWidth > 0)
             {
                  lstrcpy(logfile, temptext);
             }
             else
             {
                 dataOK = FALSE;
             }
             tempint = GetDlgItemInt(hdlg, FFT_PASSES, NULL, FALSE);
             if (tempint > 0 && tempint < maxruns+1)
             {
                 FFTpasses = tempint;
                 if (tempPasses != FFTpasses) isChanged = TRUE;
             }
             else
             {
                 dataOK = FALSE;
             }
             tempint = GetDlgItemInt(hdlg, MAXRUNSECS, NULL, FALSE);
             if (tempint > 0)
             {
                 maxSecs = tempint;
             }
             else
             {
                 dataOK = FALSE;
             }        
             if (dataOK)
             {
                if (isChanged)
                {
                    for (i=0; i<FFTsizes; i++)
                    {
                        for (j=0; j<maxruns; j++)
                        {
                            SPFFT[i][j] = 0;
                            DPFFT[i][j] = 0;
                        }
                    }
                }
                EndDialog( hdlg, TRUE );
                return (TRUE);
             }
             else
             {
                MessageBox(NULL, " Parameters wrong. Try again", "Error", MB_ICONWARNING | MB_OK);
                EndDialog( hdlg, TRUE );
                return (FALSE);
             }                 
        }
        else if( LOWORD( wparam ) == IDCANCEL )
        {
            if (isChanged)
            {
                for (i=0; i<FFTsizes; i++)
                {
                    for (j=0; j<maxruns; j++)
                    {
                        SPFFT[i][j] = 0;
                        DPFFT[i][j] = 0;
                    }
                }
            }
            EndDialog( hdlg, TRUE );    
            return (FALSE);
        }
        else if( HIWORD( wparam ) == CBN_SELCHANGE)
        {
            switch (LOWORD (wparam))
            {
                case MINFFTSIZE:
                  minb = (int)SendDlgItemMessage(hdlg, MINFFTSIZE, CB_GETCURSEL, 0, 0);
                  if (minb != minFFTRun) isChanged = TRUE;  
                  break;
        
                case MAXFFTSIZE:
                  maxb = (int)SendDlgItemMessage(hdlg, MAXFFTSIZE, CB_GETCURSEL, 0, 0);
                  if (maxb != maxFFTRun) isChanged = TRUE;
                  break;

                case MAXSCALE:
                  maxScalePos = (int)SendDlgItemMessage(hdlg, MAXSCALE, CB_GETCURSEL, 0, 0);
                  break;
            }
            if (isChanged)
            {
                for (i=0; i<FFTsizes; i++)
                {
                    for (j=0; j<maxruns; j++)
                    {
                        SPFFT[i][j] = 0;
                        DPFFT[i][j] = 0;
                    }
                }
            }
            UpdateCombos(hdlg, minb, maxb);
        }
     }
     return( FALSE );

} // WriteReadGraph


BOOL  drawGraphs(int nw, int nr, HDC hdc)
{
    HBRUSH hbr;
    HPEN hpen;

    int  thissize;
    char thisdata[200];
    TEXTMETRIC tmt;
    int     charHeight;
    int     charWidth;
    PLOGFONT  plf;
    HFONT   hfont;
    int   nPtSize;   
    int   text2 =  5;   
    int   yTop1 =    44;  // 20
    int   yBottom1 = 330; // 340
    int   yInc1;
    double   scale1;
    double   mMax;
    double   sMax;
    int   height1;
    int   wide1;
    int   xStart = 135;  // 140;
    int   xEnd = 615; // 620
    int   yBottom2;
    int   yTitle1 = 400;
    int   i, j, m, g;
    int   pixelsOn;
    int   incH;
    int   incW;
    char  sSP[10];
    char  sDP[10];
    double minSPc;
    double minDPc;
    double SPFFTU[11][20];
    double DPFFTU[11][20];
    BOOL    higher;


    yBottom2 = butT;

    height1 = yBottom1 - yTop1;
    incH = height1 / 11;
    yInc1 = height1/10;
    wide1 = xEnd - xStart;
    incW = wide1 / 10;

    GetClientRect(mainHWND, &recwin);
    j = (recwin.right + 3) / 4;
    xtraWide = j * 4  - recwin.right;
    if (xtraWide == 1) recwin.right = recwin.right - 3;
    if (xtraWide == 2) recwin.right = recwin.right - 2;
    if (xtraWide == 3) recwin.right = recwin.right - 1;


    // Max Scale
    for (i=0; i<FFTsizes; i++)
    {
        for (j=0; j<maxruns; j++)
        {
            SPFFTU[i][j] = SPFFT[i][j];
            DPFFTU[i][j] = DPFFT[i][j];
            if (maxScale > 0)
            {
                if (SPFFTU[i][j] > maxScale) SPFFTU[i][j] = maxScale;
                if (DPFFTU[i][j] > maxScale) DPFFTU[i][j] = maxScale;
            }
        }
    }

    // Max times and average millisecs/K

    for (i=0; i<FFTsizes; i++)
    {
        miniSP[i] = 0;
        miniDP[i] = 0;
    }        
    mMax = 0;
    for (i=minFFTRun; i<maxFFTRun+1; i++)
    {
        minSPc = 10000000;
        minDPc = 10000000;
        for (j=0; j<FFTpasses; j++)
        {
            if (SPFFTU[i][j] > mMax) mMax = SPFFTU[i][j];
            if (DPFFTU[i][j] > mMax) mMax = DPFFTU[i][j];
            if (SPFFT[i][j] > 0 && SPFFT[i][j] < minSPc) minSPc = SPFFT[i][j]; 
            if (DPFFT[i][j] > 0 && DPFFT[i][j] < minDPc) minDPc = DPFFT[i][j]; 
        }
        if (minSPc < 10000000) miniSP[i] = minSPc * (double)numberFFTs[i];
        if (minDPc < 10000000) miniDP[i] = minDPc * (double)numberFFTs[i];
    }
    if (maxScale > 0) mMax = maxScale;

    // adjust scale

    sMax = 0.001;
    for (j=1; j<11; j++)
    {
        for (i=1; i<11; i++)
        {
            if (mMax < sMax * (double)i || mMax == sMax * (double)i)
            {
               scale1 = sMax * (double)i;
               i = 11;
               j = 11;
            }
        }
        sMax = sMax * 10;
    }


    // Fill most of screen white

    hbr = CreateSolidBrush(RGB(255,255,255));  
    SelectObject(hdc, hbr);
    hpen = CreatePen(PS_SOLID, 0, RGB(255,255,255));
    SelectObject(hdc, hpen);
    Rectangle(hdc, recwin.left, recwin.top, recwin.right, butT - 1);
    Rectangle(hdc, recwin.left, butT-1, butL - 1, butT+butH+1);
    Rectangle(hdc, recwin.left, butT+butH+1, recwin.right, recwin.bottom);
    DeleteObject(hpen);
    
    // Text black on white

    SetBkColor(hdc, RGB(255,255,255));
    SetTextColor(hdc, RGB(0,0,0)); 

    // Heading Font
    
    plf = (PLOGFONT) LocalAlloc(LPTR, sizeof(LOGFONT));
    nPtSize = 12; // 10
    plf->lfHeight = -MulDiv(nPtSize, GetDeviceCaps(hdc, LOGPIXELSY), 72);
    strcpy(plf->lfFaceName, "Times New Roman");
    hfont = CreateFontIndirect(plf);
    SelectObject(hdc, hfont);

    GetTextMetrics(hdc, &tmt);
    charHeight = tmt.tmHeight;
    charWidth  = tmt.tmAveCharWidth;

    if (testRunning)
    {
        sprintf(thisdata, "Test Running - Press Esc Key To Kill If Running Time Too Long                  ");
    }
    else
    {
        sprintf(thisdata, "Scott's & Roy's FFTGraf Benchmark %s                                      ", version);
    }
    thissize =  strlen(thisdata);
    ExtTextOut(hdc, xStart+80, 3, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);


    LocalFree((HLOCAL) plf);
    DeleteObject(hfont);

    // Font
    
    plf = (PLOGFONT) LocalAlloc(LPTR, sizeof(LOGFONT));
    nPtSize = 8;
    plf->lfHeight = -MulDiv(nPtSize, GetDeviceCaps(hdc, LOGPIXELSY), 72);
    strcpy(plf->lfFaceName, "Courier New");
    hfont = CreateFontIndirect(plf);
    SelectObject(hdc, hfont);

    GetTextMetrics(hdc, &tmt);
    charHeight = tmt.tmHeight;
    charWidth  = tmt.tmAveCharWidth;

    hpen = CreatePen(PS_SOLID, 0, RGB(0,0,0));
    SelectObject(hdc, hpen);
     
    // Headings

    if (!compareTwo)
    {
        sprintf(thisdata, "Memory used = 16 x SP size or 28 x DP size, maximum 28 MB"); 
        thissize =  strlen(thisdata);
        ExtTextOut(hdc, xStart+50, charHeight+10, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Size   Min Millisecs  ");
    }
    else
    {
        sprintf(thisdata, "  Speed Comparison of running times  >1 A is faster      ");
        thissize =  strlen(thisdata);
        ExtTextOut(hdc, xStart+50, charHeight+10, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Size   KBytes Used  ");
    }
    thissize =  strlen(thisdata);
    ExtTextOut(hdc, 10, charHeight-2, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    sprintf(thisdata, " K     SP     DP     ");
    thissize =  strlen(thisdata);
    ExtTextOut(hdc, 10, charHeight*2-2, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);


    // X titles

    if (!compareTwo)
    {
        sprintf(thisdata, "Millisecs / K Size  ");
    }
    else
    {
        sprintf(thisdata, "Millisecs Ratio B/A ");
    }
    thissize = strlen(thisdata);;
    ExtTextOut(hdc, xStart + 150, yBottom1+charHeight+1, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);


    // Variables

    if (!compareTwo)
    {
        sprintf(thisdata, " %d Passes, Min %dK, Max %dK, Max Seconds %d    ", FFTpasses,
                                  numberFFTs[minFFTRun], numberFFTs[maxFFTRun], maxSecs);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, 15, yBottom2, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    }
    else
    {
        sprintf(thisdata, "A %s  ", config1);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, 15, yBottom2, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    }
    if (!compareTwo)
    {
       sprintf(thisdata, " Memory Size %d KBytes     ", ramKB);
    }
    else
    {
       sprintf(thisdata, "B %s  ", config2);
    }                                     
    thissize = strlen(thisdata);
    ExtTextOut(hdc, 15, yBottom2+charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
     

    // Colour keys

    sprintf(thisdata, "Key ");
    thissize = 4;
    ExtTextOut(hdc, 5, yBottom1+5, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    SetBkColor(hdc, RGB(255,0,0));
    SetTextColor(hdc, RGB(255,255,255)); 
    sprintf(thisdata, "  SP");
    thissize = 4;
    ExtTextOut(hdc, 45, yBottom1+5, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    SetBkColor(hdc, RGB(0,0,255));
    sprintf(thisdata, "  DP");
    thissize = 4;
    ExtTextOut(hdc, 95, yBottom1+5, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    SetTextColor(hdc, RGB(0,0,0)); 


    // Graph Box
    
    MoveToEx(hdc, xStart, yTop1, NULL);
    LineTo(hdc, xStart, yBottom1);
    LineTo(hdc, xEnd, yBottom1);
    LineTo(hdc, xEnd, yTop1);
    LineTo(hdc, xStart-3, yTop1);

    SetBkColor(hdc, RGB(255,255,255));

    // Size & results

    j = 0;
    for (i=yTop1-2; i<yBottom1-2; i=i+incH)
    {
        sprintf(thisdata, "%4d", numberFFTs[j]);
        if (!compareTwo)
        {
            SPres = miniSP[j];
            sprintf(sSP, "%7.0f", SPres);
            if (SPres < 100) sprintf(sSP, "%7.1f", SPres);
            if (SPres < 10) sprintf(sSP, "%7.2f", SPres);
            if (SPres < 1) sprintf(sSP, "%7.3f", SPres);
            if (SPres == 0) sprintf(sSP, "       ");
        }
        else
        {
            SPres = 16.0 * (double)numberFFTs[j];
            sprintf(sSP, "%7.0f", SPres);
        }

        if (!compareTwo)
        {
            DPres = miniDP[j];
            sprintf(sDP, "%7.0f", DPres);
            if (DPres < 100) sprintf(sDP, "%7.1f", DPres);
            if (DPres < 10) sprintf(sDP, "%7.2f", DPres);
            if (DPres < 1) sprintf(sDP, "%7.3f", DPres);
            if (DPres == 0) sprintf(sDP, "       ");
        }
        else
        {
            DPres = 28.0 * (double)numberFFTs[j];
            sprintf(sDP, "%7.0f", DPres);
        }

        if (flash1 == 1 && runningTest == j)
        {
            SetBkColor(hdc, RGB(255,0,0));
            SetTextColor(hdc, RGB(255,255,255)); 
        }
        else if (flash1 == 2 && runningTest == j)
        {
            SetBkColor(hdc, RGB(0,255,255));
            SetTextColor(hdc, RGB(255,0,0)); 
        }
        else
        {
            SetBkColor(hdc, RGB(255,255,255));        
            SetTextColor(hdc, RGB(0,0,0)); 
        }  
        strcat(thisdata, sSP);        
        thissize = strlen(thisdata);
        ExtTextOut(hdc, 0, i, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        SetBkColor(hdc, RGB(255,255,255));        
        SetTextColor(hdc, RGB(0,0,0)); 
        if (flash2 == 1 && runningTest == j)
        {
            SetBkColor(hdc, RGB(0,0,255));
            SetTextColor(hdc, RGB(255,255,255)); 
        }
        else if (flash2 == 2 && runningTest == j)
        {
            SetBkColor(hdc, RGB(0,255,255));
            SetTextColor(hdc, RGB(255,0,0)); 
        }
        else
        {
            SetBkColor(hdc, RGB(255,255,255));        
            SetTextColor(hdc, RGB(0,0,0)); 
        }
        sprintf(thisdata, "           "); // - 1 space
        strcat(thisdata, sDP);        
        thissize = strlen(thisdata);
        ExtTextOut(hdc, 0, i+incH/2, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        SetBkColor(hdc, RGB(255,255,255));        
        SetTextColor(hdc, RGB(0,0,0)); 
        j = j + 1;
    }

    // Scale

    sprintf(thisdata, "0");
    thissize = strlen(thisdata);;
    ExtTextOut(hdc, xStart-charWidth / 2, yBottom1+1, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    
    j = 1;
    for (i=xStart; i<xEnd-incW; i=i+incW)
    {
        m = 0;
        sprintf(thisdata, "%d", (int)(scale1 * j / 10));
        if (scale1 < 10) sprintf(thisdata, "%3.1f", (scale1 * j) / 10);
        if (scale1 < 1) sprintf(thisdata, "%4.2f", (scale1 * j) / 10);
        if (scale1 < 0.1) sprintf(thisdata, "%5.3f", (scale1 * j) / 10);
        if (scale1 < 0.01) sprintf(thisdata, "%6.4f", (scale1 * j) / 10);
        thissize = strlen(thisdata);
        if (thissize == 1) m = 1;
        if (thissize > 2) m = -1;
        ExtTextOut(hdc, i+incW- charWidth * (thissize - 1) - m * charWidth / 2,
           yBottom1+1, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        j = j + 1;
    }

    m = 0;
    sprintf(thisdata, "%d", (int)scale1); 
    if (scale1 < 10) sprintf(thisdata, "%3.1f", scale1);
    if (scale1 < 1) sprintf(thisdata, "%4.2f", scale1);
    if (scale1 < 0.1) sprintf(thisdata, "%5.3f", scale1);
    if (scale1 < 0.01) sprintf(thisdata, "%6.4f", scale1);
    thissize = strlen(thisdata);
    if (thissize == 1) m = 1;
    ExtTextOut(hdc, xEnd - charWidth * (thissize - 1) - m * charWidth / 2,
      yBottom1+1, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    // Grid lines

    for (i=1; i<10; i++)
    {
       MoveToEx(hdc, xStart + i * incW, yBottom1+1, NULL);
       LineTo(hdc, xStart + i * incW, yTop1);
    }

    DeleteObject(hbr);
    DeleteObject(hpen);

    // SP graphs Red
    
    hpen = CreatePen(PS_SOLID, 0, RGB(255,0,0));
    SelectObject(hdc, hpen);
    g = 1; // 4

    for (i=0; i<nw; i++)
    {
        higher = FALSE;
        for (j=0; j<maxruns; j++)
        {
            pixelsOn = (int)((double)wide1 / scale1 * SPFFTU[i][j]);
            MoveToEx(hdc, xStart, yTop1+i*incH+g+j, NULL);
            LineTo(hdc, xStart+pixelsOn, yTop1+i*incH+g+j);
            if (SPFFT[i][j] > SPFFTU[i][j]) higher = TRUE;
        }
        if (higher)
        {
            SetTextColor(hdc, RGB(255,0,0)); 
            ExtTextOut(hdc, xEnd, yTop1+incH*i-3, ETO_CLIPPED, &recwin, ">", 1, (LPINT) NULL);
            SetTextColor(hdc, RGB(0,0,0));
        }  
    }
    DeleteObject(hpen);

    // DP graphs Blue
    
    hpen = CreatePen(PS_SOLID, 0, RGB(0,0,255));
    SelectObject(hdc, hpen);
    g = 14; // 17
    for (i=0; i<nr; i++)
    {
        higher = FALSE;
        for (j=0; j<maxruns; j++)
        {
            pixelsOn = (int)((double)wide1 / scale1 * DPFFTU[i][j]);
            MoveToEx(hdc, xStart, yTop1+i*incH+g+j, NULL);
            LineTo(hdc, xStart+pixelsOn, yTop1+i*incH+g+j);
            if (DPFFT[i][j] > DPFFTU[i][j]) higher = TRUE;
        }
        if (higher)
        {
            SetTextColor(hdc, RGB(0,0,255)); 
            ExtTextOut(hdc, xEnd, yTop1+incH*i+incH/2-3, ETO_CLIPPED, &recwin, ">", 1, (LPINT) NULL);
            SetTextColor(hdc, RGB(0,0,0));
        }  
    }
    DeleteObject(hpen);

    hpen = CreatePen(PS_SOLID, 0, RGB(255,255,255));
    SelectObject(hdc, hpen);


    // Date and Configuration - bottom

    if (!compareTwo)
    {
        sprintf(thisdata, "%s ", configData1);
    }
    else
    {
        sprintf(thisdata, "A %s ", configData1);
    }        
    if (!displayOnly)
    {
        local_time();
    }
    if (!compareTwo)
    {
        strncat(thisdata, timeday, strlen(timeday)-1);
    }
    thissize = strlen(thisdata);
    ExtTextOut(hdc, 15, yBottom2 + charHeight*2, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    
    if (!compareTwo)
    {
        sprintf(thisdata, "%s ", configData2);
    }
    else
    {
        sprintf(thisdata, "B %s ", configData2);
    }
    thissize = strlen(thisdata);
    ExtTextOut(hdc, 15, yBottom2 + charHeight*3, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    DeleteObject(hpen);
    LocalFree((HLOCAL) plf);
    DeleteObject(hfont);

    ReleaseDC(mainHWND, hdc); 
    return TRUE;

} // drawGraphs


BOOL reView()
{
    HDC   hdc;
    
    SetWindowText(mainHWND, " FFT Graph Benchmark  ");  
    hdc = GetDC(mainHWND);
    drawGraphs(FFTsizes, FFTsizes, hdc);
    ReleaseDC(mainHWND, hdc); 
    return TRUE;
}


BOOL saveGrafTxt()
{
     FILE *outfile;
     int i, j;
     char  sSP[10];
     char  sDP[10];

     SetCurrentDirectory(currentDir);
     outfile = fopen(logfile,"a+");
     if (outfile == NULL)
     {
         MessageBox(NULL, "Cannot open log file", logfile,
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }
     fprintf (outfile, "\n\n ###########################################################\n");
     fprintf (outfile, "     FFTGraf Test %s %s \n", version, timeday1);
     fprintf(outfile, "      By Roy Longbottom using Scott Taylor's FFT code\n\n");
     fprintf(outfile, " %s\n", configData1);
     fprintf(outfile, " %s\n", configData2);
     fprintf(outfile, " %s\n", configData3);
     fprintf(outfile, "  %d Passes Min %d K Max %d K Max Seconds %d\n\n", FFTpasses,
                       numberFFTs[minFFTRun], numberFFTs[maxFFTRun], maxSecs);
   
     fprintf (outfile, " Size  Single Precision FFTs\n");
     fprintf (outfile, "    K  Millisecond each pass\n"); 

     for (i=minFFTRun; i<maxFFTRun+1; i++) 
     {
        fprintf (outfile, "%5d", numberFFTs[i]);
        for (j=0; j<FFTpasses; j++)
        {
            SPres = SPFFT[i][j] * (double)numberFFTs[i];
            sprintf(sSP, "%7.0f", SPres);
            if (SPres < 100) sprintf(sSP, "%7.1f", SPres);
            if (SPres < 10) sprintf(sSP, "%7.2f", SPres);
            if (SPres < 1) sprintf(sSP, "%7.3f", SPres);
            if (SPres == 0) sprintf(sSP, "       ");
            fprintf (outfile, "%s", sSP);
        }
        fprintf (outfile, "\n");
     }
     fprintf (outfile, "\n Size  Double Precision FFTs\n");
     fprintf (outfile, "    K  Millisecond each pass\n"); 
    
     for (i=minFFTRun; i<maxFFTRun+1; i++)
     {
        fprintf (outfile, "%5d", numberFFTs[i]);
        for (j=0; j<FFTpasses; j++)
        {
            DPres = DPFFT[i][j] * (double)numberFFTs[i];
            sprintf(sDP, "%7.0f", DPres);
            if (DPres < 100) sprintf(sDP, "%7.1f", DPres);
            if (DPres < 10) sprintf(sDP, "%7.2f", DPres);
            if (DPres < 1) sprintf(sDP, "%7.3f", DPres);
            if (DPres == 0) sprintf(sDP, "       ");
            fprintf (outfile, "%s", sDP);
        }
        fprintf (outfile, "\n");
     }
     fprintf (outfile, "\n%s\n", checkSP);
     fprintf (outfile, "%s\n", checkDP);
     local_time();
     fprintf (outfile, "\n               End FFT Test %s\n", timeday);
     fflush (outfile);
     fclose(outfile);
     return TRUE;

} // saveGrafTxt 


BOOL  saveBmp()
{

    HANDLE            hFile;
    LPBITMAPINFO      lpbmi = NULL;
    BITMAPFILEHEADER  bfh;
    int              dataLength;
    HDC               hdc = NULL;
    int              fileLength;
    DWORD             BytesWritten;
    char             *outBytes;
    BYTE              b;
    int              i, j, n;
    DWORD             g1, g2;
    HDC     hdcImage = NULL;
    HBITMAP hbmpImage = NULL;
    BITMAP  bmpImage;

    PALETTEENTRY Vga16Palette[16] =
    {
        { 0,   0,   0,    0 },
        { 0x80,0,   0,    0 },
        { 0,   0x80,0,    0 },
        { 0x80,0x80,0,    0 },
        { 0,   0,   0x80, 0 },
        { 0x80,0,   0x80, 0 },
        { 0,   0x80,0x80, 0 },
        { 0x80,0x80,0x80, 0 },
        { 0xC0,0xC0,0xC0, 0 },
        { 0xFF,0,   0,    0 },
        { 0,   0xFF,0,    0 },
        { 0xFF,0xFF,0,    0 },
        { 0,   0,   0xFF, 0 },
        { 0xFF,0,   0xFF, 0 },
        { 0,   0xFF,0xFF, 0 },
        { 0xFF,0xFF,0xFF, 0 }
    };
    hdc = GetDC(mainHWND);

    // Allocate and fill out a BITMAPINFO struct for the new DIB

    dataLength = (recwin.right * recwin.bottom) / 2;

    outBytes = (char *) malloc(dataLength);
    if (outBytes == NULL)
    {
        MessageBox(mainHWND, "Cannot allocate memory",
               bmpfile, MB_ICONINFORMATION | MB_OK);
        ReleaseDC(mainHWND, hdc);
        return FALSE;
    }

    UpdateWindow(mainHWND);    

    hbmpImage = CreateCompatibleBitmap(hdc, recwin.right, recwin.bottom);
    GetObject(hbmpImage, sizeof(BITMAP), &bmpImage);
    hdcImage = CreateCompatibleDC(hdc);
    SelectObject(hdcImage, hbmpImage);
    BitBlt(hdcImage, 0, 0, recwin.right, recwin.bottom, hdc, 0, 0, SRCCOPY);
    n = 0;
    for (i=0; i<recwin.bottom; i++)
    {
        for (j=0; j<recwin.right; j=j+2)
        {
            b = 0x00;
            g1 = GetPixel(hdcImage, j+0, recwin.bottom-1-i);
            g2 = GetPixel(hdcImage, j+1, recwin.bottom-1-i);
            
            if(g1 == RGB(255,0,0)) b = b | 0xC0;
            if(g2 == RGB(255,0,0)) b = b | 0x0C;
            if(g1 == RGB(0,0,255)) b = b | 0x90;
            if(g2 == RGB(0,0,255)) b = b | 0x09;
            if(g1 == RGB(255,255,255)) b = b | 0xF0;
            if(g2 == RGB(255,255,255)) b = b | 0x0F;
            outBytes[n] = b;
            n=n+1;
        }
    }
    DeleteDC(hdcImage);
    DeleteObject(hbmpImage);

    lpbmi = (BITMAPINFO*) malloc( sizeof( BITMAPINFO ) + dataLength + 264 );
    if (lpbmi == NULL)
    {
        MessageBox(mainHWND, "Cannot allocate memory",
               bmpfile, MB_ICONINFORMATION | MB_OK);
        ReleaseDC(mainHWND, hdc);
        free(outBytes);
        return FALSE;
    }
    lpbmi->bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
    lpbmi->bmiHeader.biWidth = recwin.right;
    lpbmi->bmiHeader.biHeight = recwin.bottom;
    lpbmi->bmiHeader.biPlanes = 1;
    lpbmi->bmiHeader.biBitCount = 4;
    lpbmi->bmiHeader.biCompression = BI_RGB;
    lpbmi->bmiHeader.biSizeImage = dataLength;
    lpbmi->bmiHeader.biXPelsPerMeter = 0;
    lpbmi->bmiHeader.biYPelsPerMeter = 0;
    lpbmi->bmiHeader.biClrUsed = 0;
    lpbmi->bmiHeader.biClrImportant = 0;

    memcpy(lpbmi->bmiColors, Vga16Palette, 64);    
    memcpy(lpbmi->bmiColors+64, outBytes, dataLength);    
    fileLength = 54 + 64 + dataLength;

    hFile = CreateFile(bmpfile, GENERIC_WRITE, 0, NULL,
                        CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
    {
         MessageBox(mainHWND, "Cannot open file", bmpfile,
                         MB_ICONINFORMATION | MB_OK);
        ReleaseDC(mainHWND, hdc);
        free( lpbmi );
        free(outBytes);
        return FALSE;
    }
    
    // BMP file header

    bfh.bfType = 0x4d42;
    bfh.bfSize = fileLength;
    bfh.bfReserved1 = 0;
    bfh.bfReserved2 = 0;
    bfh.bfOffBits =  118;

    WriteFile( hFile, &bfh, 14, &BytesWritten, NULL );
    WriteFile( hFile, lpbmi, 104, &BytesWritten, NULL );
    WriteFile( hFile,  lpbmi->bmiColors+64, dataLength, &BytesWritten, NULL ); 

    CloseHandle( hFile ); 

    free( lpbmi );
    free(outBytes);
    ReleaseDC(mainHWND, hdc);
    
    return TRUE;
    
} // saveBmp()
    

static BOOL GetFileName()
{
    static char    filterList1[] = "File (*.txt)" \
                                        "\0" \
                                        "*.txt" \
                                        "\0\0";
    static char    filterList2[] = "File (*.bmp)" \
                                        "\0" \
                                        "*.bmp" \
                                        "\0\0";
    OPENFILENAME        of;
    int                 rc;
    
    if (loadIt)
    {
        grafFile[ 0 ] = 0;
    }
    else
    {
        strcpy(grafFile, bmpfile);
    }
    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = mainHWND;
    of.lpstrFilter = (LPSTR) filterList1;
    if (!loadIt) of.lpstrFilter = (LPSTR) filterList2;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = grafFile;
    of.nMaxFile = _MAX_PATH;
    of.lpstrTitle = NULL;
    of.Flags = OFN_HIDEREADONLY;
    of.lpstrInitialDir = currentDir;
    
    if (loadIt)
    {
        rc = GetOpenFileName( &of );
    }
    else
    {
        rc = GetSaveFileName( &of );
        strcpy(bmpfile, of.lpstrFile);
    }
   
    return( rc );
    
} // GetFileName

BOOL loadFile()
{
     HDC   hdc;
     FILE *infile;
     char buf[350];
     char buf2[350];
     char buf3[350];
     BOOL fin = FALSE;
     BOOL skip;
     int  gotGraf = 0;
     int  i, j, i1;
     char fpnumber[6];
     char v[10];
     int  mbYNC;
     int  minKB, maxKB;
     int  grafCount = 0;
     int  count2 = 0;

     infile = fopen(fileToRead,"r");
     if (infile == NULL)
     {
         MessageBox(NULL, "Cannot open graph log file", logfile,
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }

    while (!fin && fgets(buf, 120, infile) != NULL)
    {
        if (strnicmp(buf, "     FFTGraf Test", 17) == 0)
        {
            grafCount = grafCount + 1;
        }
     }
     fclose(infile);

     infile = fopen(fileToRead,"r");
     if (infile == NULL)
     {
         MessageBox(NULL, "Cannot open graph log file", logfile,
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }

    while (!fin && fgets(buf, 120, infile) != NULL)
    {
        skip = FALSE;
        if (strnicmp(buf, "     FFTGraf Test", 17) == 0)
        {
            for (i=0; i<FFTsizes; i++)
            {
                for (j=0; j<maxruns; j++)
                {
                    SPFFT[i][j] = 0;
                    DPFFT[i][j]  = 0;
                }
            }
//            sscanf(buf, "%31c%25c ", buf2, timeday);
            sscanf(buf, "%27c%s%25c ", buf2, v, timeday);
            strcat(timeday, " ");
            if ((int)strtod(v, NULL) > 1)
            {
                 MessageBox(NULL, "Use later program version to view or compare", "Not Version 1 Results",
                                         MB_ICONWARNING | MB_OK);
                 return FALSE;
            }
            count2 = count2 + 1;

            fgets(buf, 120, infile);
            fgets(buf, 120, infile);
            fgets(buf, 120, infile);
            fgets(buf, 2, infile);
            fgets(buf, 120, infile);
            strncpy(configData1, buf, strlen(buf) - 1);
            configData1[strlen(buf)-1] = 0;
            fgets(buf, 2, infile);
            fgets(buf, 120, infile);
            strncpy(configData2, buf, strlen(buf) - 1);
            configData2[strlen(buf)-1] = 0;
            fgets(buf, 1, infile);
            fgets(buf, 120, infile);
            strncpy(configData3, buf, strlen(buf) - 1);
            sscanf(buf, "%s %d", buf, &ramKB);
            configData3[strlen(buf)] = 0;
            displayOnly = TRUE;
            fscanf(infile, "%d %s %s %d %s %s %d %s %s %s %d",
                &FFTpasses, buf, buf, &minKB, buf, buf, &maxKB, buf, buf, buf, &maxSecs);
            for (i=0; i<FFTsizes; i++)
            {
                if (numberFFTs[i] == minKB) minFFTRun = i;
                if (numberFFTs[i] == maxKB) maxFFTRun = i;
            }
            fgets(buf, 120, infile);
            fgets(buf, 120, infile);
            fgets(buf, 120, infile);
            if (strnicmp(buf, " Size", 5) != 0) skip = TRUE;
            if (!skip)
            {
                fgets(buf, 120, infile);
                for (i=minFFTRun; i<maxFFTRun+1; i++)
                {
                    fscanf(infile, "%5d", &i1);
                    for (j=0; j<FFTpasses; j++)
                    {
                        fscanf(infile, "%7c", fpnumber);
                        SPFFT[i][j] = strtod(fpnumber, NULL) / (double)numberFFTs[i];
                    }
                    fgets(buf, 120, infile);  // CR
                }
                fgets(buf, 120, infile);
                fgets(buf, 120, infile);
                if (strnicmp(buf, " Size", 5) != 0) skip = TRUE;
            }
            if (!skip)
            {                
                fgets(buf, 120, infile);
                for (i=minFFTRun; i<maxFFTRun+1; i++)
                {
                    fscanf(infile, "%5d", &i1);
                    for (j=0; j<FFTpasses; j++)
                    {
                        fscanf(infile, "%7c", fpnumber);
                        DPFFT[i][j] = strtod(fpnumber, NULL) / (double)numberFFTs[i];
                    }
                    fgets(buf, 120, infile);  // CR
                }
                hdc = GetDC(mainHWND);
                drawGraphs(FFTsizes, FFTsizes, hdc);
                ReleaseDC(mainHWND, hdc); 
                sprintf(fileMessage, "Search for another graph?");
                sprintf(buf3, " Loaded graph %d of %d from file %s", count2, grafCount, fileToRead);
                if (count2 < grafCount)
                {
                    mbYNC = MessageBox(mainHWND, fileMessage, buf3,
                                        MB_ICONQUESTION | MB_YESNO);
                    if (mbYNC == IDNO)
                    {
                       fin = TRUE;
                       gotGraf = 1;
                    }
                    else
                    {
                        gotGraf = 2;
                    }
                }
                else
                {
                    fin = TRUE;
                    gotGraf = 1;
                }
            }
        }
    }
    sprintf(buf3, " Loading graph from file %s", fileToRead);
    if (gotGraf == 0)
    {
        MessageBox(NULL, "No graph found", buf3, MB_ICONWARNING | MB_OK);
    }
    else if (gotGraf == 2)
    {
        MessageBox(NULL, "No more graphs found", buf3, MB_ICONWARNING | MB_OK);
    }    
    fclose(infile);
    return TRUE;
     
} // loadFile()


// bit reverse table

void bitrev (int t, int n)
{
    int index;
    int point;
    int m;
    int p;
    for (index=0; index<n; index++)
    {
        p = 0;
        m = index;
        for (point=0; point<t; point++)
        {
            p = p + p;
            p = p + (m & 1);
            m = m >> 1;
        }
        table[index] = p;
    }
} // bit reverse table


// create exponent table

void createExpTableSP (int n, int sign)
{
    int   index;
    float const0;
    float rindex;
    float fsign;
    float pi2;
    
    fsign = (float)sign;
    pi2 = 2.0f * PISP;  
    const0 = (pi2 * fsign)  / (float)n;
    for (index=0; index<n/2; index++)
    {
        rindex = (float)index;
        cExpSP[index][I] = (float)cos (const0 * rindex);
        cExpSP[index][Q] = (float)sin (const0 * rindex);
    }
}
void createExpTableDP (int n, int sign)
{
    int   index;
    double const0;
    double rindex;
    double fsign;
    double pi2;
    
    fsign = (double)sign;
    pi2 = 2.0 * PIDP;  
    const0 = (pi2 * fsign)  / (double)n;
    for (index=0; index<n/2; index++)
    {
        rindex = (double)index;
        cExpDP[index][I] = cos (const0 * rindex);
        cExpDP[index][Q] = sin (const0 * rindex);
    }
} // create exponent table

// create data to avoid errors

void createRealDataSP (int n, float synthFreq)
{
    float  const0;
    float  rindex;
    int   index;
    
    const0 = (2.0f  * PISP * synthFreq) / (float)n;
    for (index=0; index<n; index++)
    {
        rindex = (float)index;
        cDataSP[index][I] = (float)cos (const0 * rindex);
        cDataSP[index][Q] = (float)sin (const0 * rindex);
    }
}
void createRealDataDP (int n, double synthFreq)
{
    double  const0;
    double  rindex;
    int   index;
    
    const0 = (2.0  * PIDP * synthFreq) / (double)n;
    for (index=0; index<n; index++)
    {
        rindex = (double)index;
        cDataDP[index][I] = cos (const0 * rindex);
        cDataDP[index][Q] = sin (const0 * rindex);
    }
} // create data to avoid errors


void fftSP (int t, int n)
{
  int   point;
  float  inverse_n;
  float  temp1;
  float  temp2;
  int   d;
  int   d2;
  int   index;
  int   j;
  int   k;
  int   jk;
  int   jkd;
  int   jnd2;
  register float  tempReal;
  register float  tempImag;

  inverse_n = 1.0f / (float)n;
  for (index=0; index<n; index++)
  {
    point = table[index];
    if (point >= index)
    {
      temp1 = cDataSP[index][I] * inverse_n;
      temp2 = cDataSP[index][Q] * inverse_n;
      cDataSP[index][I]   = cDataSP[point][I] * inverse_n;
      cDataSP[index][Q]   = cDataSP[point][Q] * inverse_n;
      cDataSP[point][I]   =       temp1;
      cDataSP[point][Q]   =       temp2;
   }
  }
    d = 1;
    index = 1;
    while (index <= t)
    {
      d2 = d + d;
      for (j=0; j<d; j++)
      {
        k = 0;
        jnd2 =  n / d2 * j; 
        while (k < n)
        {
          jk = j + k;
          jkd = jk + d;
          tempReal   = (cDataSP[jkd][I] * cExpSP[jnd2][I]) - (cDataSP[jkd][Q] * cExpSP[jnd2][Q]);
          tempImag   = (cDataSP[jkd][Q] * cExpSP[jnd2][I]) + (cDataSP[jkd][I] * cExpSP[jnd2][Q]);
          cDataSP[jkd][I]  = cDataSP[jk][I] - tempReal;
          cDataSP[jkd][Q]  = cDataSP[jk][Q] - tempImag;
          cDataSP[jk][I]   = cDataSP[jk][I] + tempReal;
          cDataSP[jk][Q]   = cDataSP[jk][Q] + tempImag;
          k = k + d2;
        }
      }
      index = index + 1;
      d = d2;
    }
}
void fftDP (int t, int n)
{
  int   point;
  double  inverse_n;
  double  temp1;
  double  temp2;
  int   d;
  int   d2;
  int   index;
  int   j;
  int   k;
  int   jk;
  int   jkd;
  int   jnd2;
  register double  tempReal;
  register double  tempImag;

  inverse_n = 1.0 / n;
  for (index=0; index<n; index++)
    {
    point = table[index];
    if (point >= index)
                  {
      temp1 = cDataDP[index][I] * inverse_n;
      temp2 = cDataDP[index][Q] * inverse_n;
      cDataDP[index][I]   = cDataDP[point][I] * inverse_n;
      cDataDP[index][Q]   = cDataDP[point][Q] * inverse_n;
      cDataDP[point][I]   =       temp1;
      cDataDP[point][Q]   =       temp2;
      }
    }
    d = 1;
    index = 1;
    while (index <= t)
    {
      d2 = d + d;
      for (j=0; j<d; j++)
      {
        k = 0;
        jnd2 =  n / d2 * j; 
        while (k < n)
        {
          jk = j + k;
          jkd = jk + d;
          tempReal   = (cDataDP[jkd][I] * cExpDP[jnd2][I]) - (cDataDP[jkd][Q] * cExpDP[jnd2][Q]);
          tempImag   = (cDataDP[jkd][Q] * cExpDP[jnd2][I]) + (cDataDP[jkd][I] * cExpDP[jnd2][Q]);
          cDataDP[jkd][I]  = cDataDP[jk][I] - tempReal;
          cDataDP[jkd][Q]  = cDataDP[jk][Q] - tempImag;
          cDataDP[jk][I]   = cDataDP[jk][I] + tempReal;
          cDataDP[jk][Q]   = cDataDP[jk][Q] + tempImag;
          k = k + d2;
        }
      }
      index = index + 1;
      d = d2;
    }
} // fftDP

void SPCalculate()
{
    HDC   hdc;
    int i, j, t, n, sign;
    float synthFreq;
    int  Tmin, Tmax;
    
    keyPress = GetAsyncKeyState(VK_ESCAPE);

    testRunning = TRUE;

    if (tpass == 0 && jpass == 0)
    {
        for (i=0; i<FFTsizes; i++)
        {
            for (j=0; j<maxruns; j++)
            {
                SPFFT[i][j] = 0;
                DPFFT[i][j] = 0;
            }
        }
    }
    Tmin = minFFTRun + 10;
    Tmax = maxFFTRun + 10;
    
    t = tpass + Tmin;
    {
        runningTest = t - 10;
        n = (int)pow(2,(double)t);    
        nSP = n;    
         bitrev (t, n); 
        sign = -1; 
        createExpTableSP(n, sign);
        synthFreq = (float)n/16;
        j = jpass;
        {
            if (flash1 == 1)
            {
                flash1 = 2;
            }
            else
            {
                flash1 = 1;
            }
            createRealDataSP(n, synthFreq);
            getStartTime();    
            fftSP (t, n); 
            getRunSecs();
            SPFFT[runningTest][j] = runSecs * 1000 / (double)(n / 1024);
            hdc = GetDC(mainHWND);
            drawGraphs(FFTsizes, FFTsizes, hdc);
            ReleaseDC(mainHWND, hdc); 
            if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
            {
                escPressed = TRUE;
            }
        }
        if (!escPressed &&  runSecs * 2 > (double)maxSecs
                                && t != Tmax && j == FFTpasses - 1)
        {
            sprintf (amsg, "Next tests likely to take >%d seconds\n"
                           "Continue or Cancel next tests?", maxSecs); 
            if (MessageBox(mainHWND, amsg, "Test Time Too Long?",
                    MB_ICONQUESTION | MB_OKCANCEL) == IDCANCEL)
             {
                 tpass = maxFFTRun + 1;
             }
        }
    }    
} // SPCalculate()

void DPCalculate()
{
    HDC   hdc;
    int i, j, t, n, sign;
    float synthFreq;
    int  Tmin, Tmax;
    
    keyPress = GetAsyncKeyState(VK_ESCAPE);

    testRunning = TRUE;
    testRunning = TRUE;

    if (tpass == 0 && jpass == 0)
    {
        for (i=0; i<FFTsizes; i++)
        {
            for (j=0; j<FFTpasses; j++)
            {
                DPFFT[i][j] = 0;
            }
        }
    }
    
    Tmin = minFFTRun + 10;
    Tmax = maxFFTRun + 10;
    
    t = tpass + Tmin;
    {
        runningTest = t - 10;
        n = (int)pow(2,(double)t);    
        nDP = n;    
         bitrev (t, n); 
        sign = -1; 
        createExpTableDP(n, sign);
        synthFreq = (float)n/16;
        j = jpass;
        {
            if (flash2 == 1)
            {
                flash2 = 2;
            }
            else
            {
                flash2 = 1;
            }
            createRealDataDP(n, synthFreq);
            getStartTime();    
            fftDP (t, n); 
            getRunSecs();
            DPFFT[runningTest][j] = runSecs * 1000 / (double)(n / 1024);
            hdc = GetDC(mainHWND);
            drawGraphs(FFTsizes, FFTsizes, hdc);
            ReleaseDC(mainHWND, hdc); 
            if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
            {
                escPressed = TRUE;
            }
        }
        if (!escPressed &&  runSecs * 2 > (double)maxSecs
                          && t != Tmax && j == FFTpasses - 1)
        {
            sprintf (amsg, "Next tests likely to take >%d seconds\n"
                           "Continue or Cancel next tests?", maxSecs); 
            if (MessageBox(mainHWND, amsg, "Test Time Too Long?",
                    MB_ICONQUESTION | MB_OKCANCEL) == IDCANCEL)
             {
                 tpass = maxFFTRun + 1;
             }
        }
    }    
} // DPCalculate()


void Compare0()
{
    int i, j;
    
    for (i=0; i<FFTsizes; i++)
    {
        for (j=0; j<maxruns; j++)
        {
            SP1[i][j] = 0;
            SP2[i][j] = 0;
            DP1[i][j] = 0;
            DP2[i][j] = 0;
        }
    }
}

void Compare1()
{
    int i, j;

    for (i=0; i<FFTsizes; i++)
    {
        for (j=0; j<maxruns; j++)
        {
            SP1[i][j] = SPFFT[i][j];
            DP1[i][j] = DPFFT[i][j];
        }
        miniSPA[i] = miniSP[i];
        miniDPA[i] = miniDP[i];
    }
    if (strlen(configData1) < 49)
    {
        strcpy(config1, configData1);
    }
    else
    {
        strncpy(config1, configData1, 48);
    } 
    strcpy(config3, configData2); 
}

void Compare2()
{
    int i, j;
    HDC   hdc;
    
    for (i=0; i<FFTsizes; i++)
    {
        for (j=0; j<maxruns; j++)
        {
            SP2[i][j] = SPFFT[i][j];
            DP2[i][j] = DPFFT[i][j];
            if (SP1[i][j] * SP2[i][j] > 0)
            {
                SPFFT[i][j] = SP2[i][j] / SP1[i][j];
            }
            else
            {
                SPFFT[i][j] = 0;
            }
            if (DP1[i][j] * DP2[i][j] > 0)
            {
                DPFFT[i][j] = DP2[i][j] / DP1[i][j];
            }
            else
            {
                DPFFT[i][j] = 0;
            }
        }
        miniSPB[i] = miniSP[i];
        miniDPB[i] = miniDP[i];
    }
    if (strlen(configData1) < 49)
    {
        strcpy(config2, configData1);
    }
    else
    {
        strncpy(config2, configData1, 48);
    } 
    strcpy(configData1, config3); 
    hdc = GetDC(mainHWND);
    compareTwo = TRUE;
    drawGraphs(FFTsizes, FFTsizes, hdc);
    ReleaseDC(mainHWND, hdc); 
}

BOOL compareTxt()
{
     FILE *outfile;
     int i, j, n;
     double ratio;
     double minSP1[11];
     double minDP1[11];
     double maxy;
     double maxz;
     double miny;
     int  pos1;
     int  pos2;
     int  pos3;
     int  pos4;
     int  pos5;

     SetCurrentDirectory(currentDir);
     outfile = fopen("Compare2.txt","a+");
     if (outfile == NULL)
     {
         MessageBox(NULL, "Cannot open compare file", "Compare2.txt",
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }
     fprintf (outfile, "\n\n ###########################################################\n");
     fprintf (outfile, "                 FFTGraf Test Speed Comparison\n\n");
     fprintf (outfile, " A %s\n", config1);
     fprintf (outfile, " B %s\n", config2);
     fprintf (outfile, " A %s\n", configData1);
     fprintf (outfile, " B %s\n\n", configData2);

     fprintf (outfile, " Minimum Millisecs Ratio B/A. If > 1.0 A is faster\n\n");

     maxy = 0;
     miny = 1000000;
     for (i=0; i<FFTsizes; i++)
     {
        if (miniSPA[i] > 0)
        {
            minSP1[i] = miniSPB[i] / miniSPA[i];
            if (minSP1[i] > maxy) maxy = minSP1[i]; 
            if (minSP1[i] > 0 && minSP1[i] < miny) miny = minSP1[i]; 
        }
        else
        {
            minSP1[i] = 0;
        }
        if (miniDPA[i] > 0)
        {
            minDP1[i] = miniDPB[i] / miniDPA[i];
            if (minDP1[i] > maxy) maxy = minDP1[i]; 
            if (minSP1[i] > 0 && minDP1[i] < miny) miny = minDP1[i]; 
        }
        else
        {
            minDP1[i] = 0;
        }
     }
     maxz = maxy;

     fprintf (outfile, " Minimum Ratio =  %6.4f. Maximum Ratio = %6.4f\n\n", miny, maxy);
          
     if (maxy > 0) maxy = 60 / maxy;
     pos1 = (int)(maxy + 0.5); 
     pos2 = (int)(2 * maxy + 0.5);
     pos3 = (int)(3 * maxy + 0.5);
     pos4 = (int)(4 * maxy + 0.5);
     pos5 = (int)(5 * maxy + 0.5);

     fprintf (outfile, " Single Precision FFTs\n");
     fprintf (outfile, " Memory 16KB, 32KB, 64KB, 128KB, 256KB, 512KB, 1MB ... 16MB\n\n");

     for (i=0; i<FFTsizes; i++)
     {
        ratio = minSP1[i];
        n = (int)(ratio * maxy + 0.5);
        if (ratio > 9.999999)
        {
            fprintf (outfile, " >=10  ");
        }
        else if (ratio == 0)
        {
            fprintf (outfile, " N/A  ");
        }
        else
        {
            fprintf (outfile, " %4.2f  ", ratio);
        }
        for (j=1; j<n+1; j++)
        {
            if (maxz > 9.999999)
            {
                fprintf (outfile, "#");
            }
            else if (j == pos1)
            {
                fprintf (outfile, "1");
            }
            else if (j == pos2)
            {
                fprintf (outfile, "2");
            }
            else if (j == pos3)
            {
                fprintf (outfile, "3");
            }
            else if (j == pos4)
            {
                fprintf (outfile, "4");
            }
            else if (j == pos5)
            {
                fprintf (outfile, "5");
            }
            else
            {
                fprintf (outfile, "#");
            }
        }
        fprintf (outfile, "\n");
     }
     fprintf (outfile, "\n");
 
     fprintf (outfile, "\n Double Precision FFTs\n");
     fprintf (outfile, " Memory 28KB, 56KB, 112KB, 224KB, 448KB, 896KB ... 28MB\n\n");
     
     for (i=0; i<FFTsizes; i++)
     {
        ratio = minDP1[i];
        n = (int)(ratio * maxy + 0.5);
        if (ratio > 9.999999)
        {
            fprintf (outfile, " >=10  ");
        }
        else if (ratio == 0)
        {
            fprintf (outfile, " N/A  ");
        }
        else
        {
            fprintf (outfile, " %4.2f  ", ratio);
        }
        for (j=1; j<n+1; j++)
        {
            if (maxz > 9.999999)
            {
                fprintf (outfile, "#");
            }
            else if (j == pos1)
            {
                fprintf (outfile, "1");
            }
            else if (j == pos2)
            {
                fprintf (outfile, "2");
            }
            else if (j == pos3)
            {
                fprintf (outfile, "3");
            }
            else if (j == pos4)
            {
                fprintf (outfile, "4");
            }
            else if (j == pos5)
            {
                fprintf (outfile, "5");
            }
            else
            {
                fprintf (outfile, "#");
            }
        }
        fprintf (outfile, "\n");
     }
     fprintf (outfile, "\n");

     fflush (outfile);
     fclose(outfile);
     return TRUE;

} // compareTxt


void checkNoiseSP()
{
    int k;
    float sqOne;
    float MaxNoise = 0.0;
    float AveNoise = 0.0;
    
    for(k=1;k<nSP;k++)
    {
        if(k != nSP/16)
        {
            sqOne = cDataSP[k][I] * cDataSP[k][I] + cDataSP[k][Q] * cDataSP[k][Q];
            if(sqOne > MaxNoise)
                    MaxNoise = sqOne;
            AveNoise = AveNoise + sqOne;
        }
    }
    AveNoise = AveNoise / (nSP-2);
    sqOne = cDataSP[nSP/16][I] * cDataSP[nSP/16][I] + cDataSP[nSP/16][Q] * cDataSP[nSP/16][Q];
    sprintf(checkSP, " Checks SP %14e  %10e  %10e", sqOne, MaxNoise, AveNoise);
}

void checkNoiseDP()
{
    int k;
    double sqOne;
    double MaxNoise = 0.0;
    double AveNoise = 0.0;
    
    for(k=1;k<nDP;k++)
    {
        if(k != nDP/16)
        {
            sqOne = cDataDP[k][I] * cDataDP[k][I] + cDataDP[k][Q] * cDataDP[k][Q];
            if(sqOne > MaxNoise) MaxNoise = sqOne;
            AveNoise = AveNoise + sqOne;
        }
    }
    AveNoise = AveNoise / (nDP-2);
    sqOne = cDataDP[nDP/16][I] * cDataDP[nDP/16][I] + cDataDP[nDP/16][Q] * cDataDP[nDP/16][Q];
    sprintf(checkDP, " Checks DP %14e  %10e  %10e", sqOne, MaxNoise, AveNoise);
}
